#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
"""
Validation method called in ``registry.register`` method


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.db import models
from django.db.models.base import ModelBase

from permission.handlers import PermissionHandler
from permission.exceptions import ValidationError

__all__ = ('validate',)

def validate(cls, model):
    """
    Does basic PermissionHandler option validation. Calls custom validation
    classmethod in the end if it is provided in cls. The signature of the 
    custom validation classmethod should be: def validate(cls, model).

    """
    # Before we can introspect models, they need to be fully loaded so that
    # inter-relations are set up correctly. We force that here.
    models.get_apps()

    # validate model class
    if not isinstance(model, ModelBase):
        # this mean the model is not subclass of models.Model
        raise ValidationError(
                'The model "%s" must be a subclass of ``models.Model``.' % model.__class__.__name__
            )

    # validate handler class
    if not issubclass(cls, PermissionHandler):
        raise ValidationError(
                'The handler "%s" must be a subclass of '
                '``fluidpermission.handlers.PermissionHandler``.' % cls.__class__.__name__
            )

    # call custom validation classmethod
    if hasattr(cls, 'validate'):
        cls.validate(model)



from __future__ import with_statement
from test_function_decorators import *
from test_method_decorators import *
from test_class_decorators import *

from django.test import TestCase
from django.http import HttpRequest
from django.views.generic import View

from permission.decorators import permission_required
from permission.decorators.class_decorators import permission_required as class_permission_required
from permission.decorators.method_decorators import permission_required as method_permission_required
from permission.decorators.function_decorators import permission_required as function_permission_required
from permission.tests.mock import MagicMock as Mock
from permission.tests.override_settings import with_apps

def viewbase(request, *args, **kwargs):
    assert isinstance(request, HttpRequest), \
        'An instance of "HttpRequest" is expected but "%s" is passed' % request

class ViewBase(View):
    def dispatch(self, request, *args, **kwargs):
        assert isinstance(self, ViewBase), \
            'An instance of "ViewBase" is expected but "%s" is passed' % self
        assert isinstance(request, HttpRequest), \
            'An instance of "HttpRequest" is expected but "%s" is passed' % request
    def get_object(self, queryset=None):
        from permission.tests.test_app.models import Article
        return Article.objects.get(pk=1)

class_permission_required = class_permission_required('test_app.add_article')
method_permission_required = method_permission_required('test_app.add_article')
function_permission_required = function_permission_required('test_app.add_article')

@with_apps('permission.tests.test_app')
class PermissionDecoratorsTestCase(TestCase):

    fixtures = ('permission_test_app.yaml',)

    def setUp(self):
        self.mock_request = Mock(spec=HttpRequest)
        self.mock_request.META = Mock()
        self.mock_request.user = Mock()
        self.mock_request.user.is_active.return_value = True
        self.mock_request.user.is_authenticated.return_value = True

    def test_function_views(self):
        from permission.tests.test_app.models import Article
        # class_decorators cannot handle
        self.assertRaises(
                AttributeError,
                class_permission_required,
                viewbase
            )
        # method_decorators can handle function as well
        method_view = method_permission_required(viewbase)
        method_view(
                self.mock_request, 
                queryset=Article.objects.all(),
                object_id=1,
            )

        # function_decorators of course can handle
        function_view = function_permission_required(viewbase)
        function_view(
                self.mock_request, 
                queryset=Article.objects.all(),
                object_id=1,
            )

    def test_method_views(self):
        from permission.tests.test_app.models import Article
        # class_decorators cannot handle
        self.assertRaises(
                AttributeError,
                class_permission_required,
                ViewBase.dispatch
            )
        # method_decorators of course can handle
        method_view = method_permission_required(ViewBase.dispatch)
        method_view(ViewBase(), self.mock_request, pk=1)

        # function_decorators cannot handle
        function_view = function_permission_required(ViewBase.dispatch)
        self.assertRaises(
                AttributeError, function_view,
                ViewBase(), self.mock_request, pk=1,
            )

    def test_class_views(self):
        from permission.tests.test_app.models import Article
        # class view of course can handle
        class_view = class_permission_required(ViewBase)
        class_view.as_view()(self.mock_request, pk=1)

        # method_decorators of course can handle
        method_view = method_permission_required(ViewBase)
        self.assertFalse(hasattr(method_view, 'as_view'))

        # function_decorators of course can handle
        function_view = function_permission_required(ViewBase)
        self.assertFalse(hasattr(method_view, 'as_view'))



# vim: set fileencoding=utf-8 :
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.test import TestCase
from django.contrib.auth.models import Permission

from permission import PermissionHandler
from permission.exceptions import ValidationError
from permission.tests.override_settings import with_apps
from permission.tests.override_settings import override_settings

@with_apps('permission.tests.test_app')
class PermissionHandlerTestCase(TestCase):

    def test_registry_instance(self):
        from permission.handlers import Registry
        from permission import registry

        self.assertTrue(isinstance(registry, Registry))

    def test_registry_register(self):
        from permission.handlers import Registry
        from permission.exceptions import AlreadyRegistered
        from permission.tests.test_app.models import Article
        registry = Registry()
        registry.register(Article, PermissionHandler)

        self.assertEqual(len(registry._registry), 1)
        self.assertTrue(isinstance(
                registry._registry[Article], 
                PermissionHandler
            ))

        # duplicate entry should raise AlreadyRegistered
        self.assertRaises(
                AlreadyRegistered,
                registry.register,
                Article, PermissionHandler
            )

    def test_registry_register_validation(self):
        class Mock(object):
            pass
        from permission.handlers import Registry
        from permission.tests.test_app.models import Article
        registry = Registry()

        with override_settings(DEBUG=True):
            # model must be a subclass of Model
            self.assertRaises(
                    ValidationError,
                    registry.register,
                    Mock, PermissionHandler
                )
            # handler must be a subclass of PermissionHandler
            self.assertRaises(
                    ValidationError,
                    registry.register,
                    Article, Mock,
                )

        # validation should not be happen in DEBUG=False
        with override_settings(DEBUG=False):
            # Insted of ValidationError, AttributeError will raise
            # which mean the models have not validated
            self.assertRaises(
                    AttributeError,
                    registry.register,
                    Mock, PermissionHandler
                )
            # Insted of ValidationError, TypeError will raise
            # which mean the handler have not validated
            self.assertRaises(
                    TypeError,
                    registry.register,
                    Article, Mock
                )

    def test_registry_unregister(self):
        from permission.handlers import Registry
        from permission.exceptions import NotRegistered
        from permission.tests.test_app.models import Article
        registry = Registry()
        registry.register(Article, PermissionHandler)

        registry.unregister(Article)
        self.assertEqual(len(registry._registry), 0)

        # non registered entry should raise NotRegistered
        self.assertRaises(NotRegistered, registry.unregister, Article)

    def test_registry_get_handlers(self):
        from permission.handlers import Registry
        from permission.tests.test_app.models import Article
        registry = Registry()
        registry.register(Article, PermissionHandler)

        handlers = registry.get_handlers('test_app.add_article')
        self.assertTrue(isinstance(handlers, tuple))
        self.assertEqual(len(handlers), 1)
        self.assertEqual(handlers[0].__class__, PermissionHandler)

        # not registered
        handlers = registry.get_handlers('auth.add_user')
        self.assertTrue(isinstance(handlers, tuple))
        self.assertEqual(len(handlers), 0)

        # unknown
        handlers = registry.get_handlers('not_registered_app.unknown_model')
        self.assertTrue(isinstance(handlers, tuple))
        self.assertEqual(len(handlers), 0)


    def test_get_app_permissions(self):
        from permission.tests.test_app.models import Article
        instance = PermissionHandler(model=Article)

        # get all permissions related to 'test_app'
        app_permissions = Permission.objects.filter(
                content_type__app_label='test_app'
            )
        app_permissions = set([u"test_app.%s" % p.codename for p in app_permissions.all()])

        self.assertEqual(instance.get_app_permissions(), app_permissions)

    def test_get_model_permissions(self):
        from permission.tests.test_app.models import Article
        instance = PermissionHandler(model=Article)

        # get all permissions related to Article
        model_permissions = Permission.objects.filter(
                content_type__app_label='test_app',
                content_type__model='article'
            )
        model_permissions = set([u"test_app.%s" % p.codename for p in model_permissions.all()])

        self.assertEqual(instance.get_model_permissions(), model_permissions)

    def test_get_permissions(self):
        class TestPermissionHandler(PermissionHandler):
            pass
        from permission.tests.test_app.models import Article
        instance = TestPermissionHandler(model=Article)

        # get_permissions should return same value as
        # get_model_permissions in default
        self.assertEqual(
                instance.get_permissions(),
                instance.get_model_permissions()
            )

    def test_get_permissions_with_permissions(self):
        class TestPermissionHandler(PermissionHandler):
            permissions = (
                    'auth.add_user', 'auth.change_user', 'auth.delete_user'
                )
        from permission.tests.test_app.models import Article
        instance = TestPermissionHandler(model=Article)

        # get_permissions should return the value specified
        self.assertEqual(
                instance.get_permissions(),
                set(['auth.add_user', 'auth.change_user', 'auth.delete_user'])
            )

    def test_get_permissions_with_get_permissions(self):
        class TestPermissionHandler(PermissionHandler):
            def get_permissions(self):
                return set([
                    'auth.add_user', 'auth.change_user', 'auth.delete_user'
                ])

        from permission.tests.test_app.models import Article
        instance = TestPermissionHandler(model=Article)

        # get_permissions should return the value specified
        self.assertEqual(
                instance.get_permissions(),
                set(['auth.add_user', 'auth.change_user', 'auth.delete_user'])
            )

    def test_get_permission_codename(self):
        from permission.tests.test_app.models import Article
        instance = PermissionHandler(model=Article)

        perm = 'app_label.name_model'
        codename = instance.get_permission_codename(perm)
        self.assertEqual(codename, 'name_model')

        perm = 'app_label.app_label.name_model'
        codename = instance.get_permission_codename(perm)
        self.assertEqual(codename, 'app_label.name_model')

        perm = 'name_model'
        codename = instance.get_permission_codename(perm)
        self.assertEqual(codename, 'name_model')

        perm = 'name'
        codename = instance.get_permission_codename(perm)
        self.assertEqual(codename, 'name')


    def test_has_perm_raise_exception(self):
        from permission.tests.test_app.models import Article
        instance = PermissionHandler(model=Article)

        self.assertRaises(
                NotImplementedError,
                instance.has_perm,
                None, None, None
            )



# vim: set fileencoding=utf-8 :
"""
Admin


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.conf import settings
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from permission.models import Role

if 'feincms' in settings.INSTALLED_APPS:
    from mptt.admin import FeinCMSModelAdmin as ModelAdmin
else:
    from mptt.admin import MPTTModelAdmin as ModelAdmin

# SimpleListFilter is only available with Django dev version.
#class PermissionListFilter(admin.SimpleListFilter):
#    title = _('permission')
#
#    parameter_name = 'permission'
#
#    def lookups(self, request, model_admin):
#        return (
#            ('view', _('View permissions')),
#            ('add', _('Add permissions')),
#            ('change', _('Change permissions')),
#            ('delete', _('Delete permissions')),
#            ('others', _('Other permissions')),
#        )
#    def queryset(self, request, queryset):
#        if self.value() == 'others':
#            qs = queryset.exclude(_permissions__codename__istartswith='view')
#            qs = qs.exclude(_permissions__codename__istartswith='add')
#            qs = qs.exclude(_permissions__codename__istartswith='change')
#            qs = qs.exclude(_permissions__codename__istartswith='delete')
#            return qs
#        else:
#            qs = queryset.filter(_permissions__codename__istartswith=self.value())
#            return qs

class PermissionRoleAdmin(ModelAdmin):
    fieldsets = (
            (None, {'fields': ('name', 'codename', 'description')}),
            (_('Roles'), {'fields': ('parent',)}),
            (_('Permissions'), {'fields': ((
                '_permissions', 'list_of_permissions', 'inherited_permissions'),)}),
            (_('Users'), {'fields': (('_users', 'list_of_users', 'inherited_users'),)}),
        )
    filter_horizontal = ('_permissions', '_users',)
    list_display = ('name', 'codename', 'description',)
    list_filter = ('_permissions__codename',)
    readonly_fields = (
            'children_roles', 
            'list_of_permissions', 'list_of_users',
            'inherited_permissions', 'inherited_users',)
    search_fields = (
            'codename', '_permissions__app_label', '_permissions__codename', 
            '_users__username', '_users__email'
        )
    
    def children_roles(self, obj):
        li = u"<li>%s</li>"
        ul = u"<ul>\n%s\n</ul>"
        row = []
        roles = obj.get_descendants()
        for role in roles.iterator():
            row.append(li % role)
        return ul % "\n".join(row)
    children_roles.allow_tags = True
    children_roles.short_description = _('Children roles')

    def inherited_permissions(self, obj):
        li = u"<li>%s</li>"
        ul = u"<ul>\n%s\n</ul>"
        row = []
        self_permission_pks = obj._permissions.values_list('id', flat=True)
        permissions = obj.permissions.exclude(pk__in=self_permission_pks)
        for perm in permissions.iterator():
            row.append(li % perm)
        return ul % "\n".join(row)
    inherited_permissions.allow_tags = True
    inherited_permissions.short_description = _('Inherited permissions')

    def inherited_users(self, obj):
        li = u"<li>%s</li>"
        ul = u"<ul>\n%s\n</ul>"
        row = []
        self_permission_pks = obj._users.values_list('id', flat=True)
        users = obj.users.exclude(pk__in=self_permission_pks)
        for perm in users.iterator():
            row.append(li % perm)
        return ul % "\n".join(row)
    inherited_users.allow_tags = True
    inherited_users.short_description = _('Inherited users')

    def list_of_permissions(self, obj):
        li = u"<li>%s</li>"
        ul = u"<ul>\n%s\n</ul>"
        row = []
        permissions = obj.permissions
        for perm in permissions.iterator():
            row.append(li % perm)
        return ul % "\n".join(row)
    list_of_permissions.allow_tags = True
    list_of_permissions.short_description = _('Current permissions')

    def list_of_users(self, obj):
        li = u"<li>%s</li>"
        ul = u"<ul>\n%s\n</ul>"
        row = []
        users = obj.users
        for perm in users.iterator():
            row.append(li % perm)
        return ul % "\n".join(row)
    list_of_users.allow_tags = True
    list_of_users.short_description = _('Current users')

admin.site.register(Role, PermissionRoleAdmin)



# vim: set fileencoding=utf-8 :
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.test import TestCase
from django.template import Context
from django.template import Template
from django.template import TemplateSyntaxError
from django.contrib.auth.models import User
from django.contrib.auth.models import Permission

from permission.tests.utils import create_user
from permission.tests.utils import create_role
from permission.tests.utils import create_permission

class PermissionTemplateTagsTestCase(TestCase):

    def test_permissionif_tag(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(user.has_perm('permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
                "{% load permission_tags %}"
                "{% permission user has 'permission.permission_templatetag_test_perm1' %}"
                "Success"
                "{% else %}"
                "Fail"
                "{% endpermission %}"
            ).render(context)

        self.assertEqual(out, "Success")
        

    def test_permissionif_tag_elif(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(user.has_perm('permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
                "{% load permission_tags %}"
                "{% permission user has 'permission.unknown_permission' %}"
                "Fail"
                "{% elpermission user has 'permission.unknown_permisson2' %}"
                "Fail"
                "{% elpermission user has 'permission.permission_templatetag_test_perm1' %}"
                "Success"
                "{% else %}"
                "Fail"
                "{% endpermission %}"
            ).render(context)

        self.assertEqual(out, "Success")
        
    def test_permissionif_tag_else(self):
        user = create_user('permission_templatetag_test_user1')
        perm = create_permission('permission_templatetag_test_perm1')

        user.user_permissions.add(perm)

        self.assertTrue(user.has_perm('permission.permission_templatetag_test_perm1'))

        context = Context({
            'user': user,
        })

        out = Template(
                "{% load permission_tags %}"
                "{% permission user has 'permission.unknown_permission' %}"
                "Fail"
                "{% else %}"
                "Success"
                "{% endpermission %}"
            ).render(context)

        self.assertEqual(out, "Success")
        

    def test_permissionif_tag_with_obj(self):
        from permission import registry
        from permission import PermissionHandler
        from permission.models import Role

        user = create_user('permission_templatetag_test_user1')
        role1 = create_role('permission_templatetag_test_role1')
        role2 = create_role('permission_templatetag_test_role2')
        perm = create_permission('permission_templatetag_test_perm1')

        class RolePermissionHandler(PermissionHandler):
            def has_perm(self, user_obj, perm, obj=None):
                if perm == 'permission.permission_templatetag_test_perm1':
                    if obj and obj.codename == 'permission_templatetag_test_role2':
                        return True
                return False
        registry.register(Role, RolePermissionHandler)

        self.assertFalse(user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertFalse(user.has_perm('permission.permission_templatetag_test_perm1', role1))
        self.assertTrue(user.has_perm('permission.permission_templatetag_test_perm1', role2))

        context = Context({
            'user': user,
            'role1': role1,
            'role2': role2,
        })

        out = Template(
                "{% load permission_tags %}"
                "{% permission user has 'permission.permission_templatetag_test_perm1' %}"
                "Fail"
                "{% elpermission user has 'permission.permission_templatetag_test_perm1' of role1 %}"
                "Fail"
                "{% elpermission user has 'permission.permission_templatetag_test_perm1' of role2 %}"
                "Success"
                "{% else %}"
                "Fail"
                "{% endpermission %}"
            ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_and(self):
        user = create_user('permission_templatetag_test_user1')
        perm1 = create_permission('permission_templatetag_test_perm1')
        perm2 = create_permission('permission_templatetag_test_perm2')

        user.user_permissions.add(perm1, perm2)

        self.assertTrue(user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertTrue(user.has_perm('permission.permission_templatetag_test_perm2'))

        context = Context({
            'user': user,
        })

        out = Template(
                "{% load permission_tags %}"
                "{% permission user has 'permission.unknown_perm' "
                "and user has 'permission.permission_templatetag_test_perm2' %}"
                "Fail"
                "{% elpermission user has 'permission.permission_templatetag_test_perm1' "
                "and user has 'permission.unknown_perm' %}"
                "Fail"
                "{% elpermission user has 'permission.permission_templatetag_test_perm1' "
                "and user has 'permission.permission_templatetag_test_perm2' %}"
                "Success"
                "{% endpermission %}"
            ).render(context)

        self.assertEqual(out, "Success")

    def test_permissionif_tag_or(self):

        user = create_user('permission_templatetag_test_user1')
        perm1 = create_permission('permission_templatetag_test_perm1')
        perm2 = create_permission('permission_templatetag_test_perm2')

        user.user_permissions.add(perm1)

        self.assertTrue(user.has_perm('permission.permission_templatetag_test_perm1'))
        self.assertFalse(user.has_perm('permission.permission_templatetag_test_perm2'))

        context = Context({
            'user': user,
        })

        out = Template(
                "{% load permission_tags %}"
                "{% permission user has 'permission.permission_templatetag_test_perm1' "
                "and user has 'permission.permission_templatetag_test_perm2' %}"
                "Fail"
                "{% elpermission user has 'permission.permission_templatetag_test_perm1' "
                "or user has 'permission.permission_templatetag_test_perm2' %}"
                "Success"
                "{% endpermission %}"
            ).render(context)

        self.assertEqual(out, "Success")



# vim: set fileencoding=utf-8 :
"""
short module explanation


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from distutils.version import StrictVersion
from django import get_version

if StrictVersion(get_version()) < '1.4':
    from django.template import Node
    from django.template import NodeList
    from django.template import VariableDoesNotExist
    from django.template import TemplateSyntaxError
    from django.template.base import TextNode
    # copied from django 1.4b
    class IfNode(Node):
        def __init__(self, conditions_nodelists):
            self.conditions_nodelists = conditions_nodelists
        def __repr__(self):
            return "<IfNode>"
        def __iter__(self):
            for _, nodelist in self.conditions_nodelists:
                for node in nodelist:
                    yield node
        @property
        def nodelist(self):
            return NodeList(node for _, nodelist in self.conditions_nodelists for node in nodelist)
        def render(self, context):
            for condition, nodelist in self.conditions_nodelists:
                if condition is not None:           # if / elif clause
                    try:
                        match = condition.eval(context)
                    except VariableDoesNotExist:
                        match = None
                else:                               # else clause
                    match = True
                if match:
                    return nodelist.render(context)
            return ''
    # copied from django 1.4b
    def parse(self, parse_until=None):
        if parse_until is None:
            parse_until = []
        nodelist = self.create_nodelist()
        while self.tokens:
            token = self.next_token()
            # Use the raw values here for TOKEN_* for a tiny performance boost.
            if token.token_type == 0: # TOKEN_TEXT
                self.extend_nodelist(nodelist, TextNode(token.contents), token)
            elif token.token_type == 1: # TOKEN_VAR
                if not token.contents:
                    self.empty_variable(token)
                filter_expression = self.compile_filter(token.contents)
                var_node = self.create_variable_node(filter_expression)
                self.extend_nodelist(nodelist, var_node, token)
            elif token.token_type == 2: # TOKEN_BLOCK
                try:
                    command = token.contents.split()[0]
                except IndexError:
                    self.empty_block_tag(token)
                if command in parse_until:
                    # put token back on token list so calling
                    # code knows why it terminated
                    self.prepend_token(token)
                    return nodelist
                # execute callback function for this tag and append
                # resulting node
                self.enter_command(command, token)
                try:
                    compile_func = self.tags[command]
                except KeyError:
                    self.invalid_block_tag(token, command, parse_until)
                try:
                    compiled_result = compile_func(self, token)
                except TemplateSyntaxError, e:
                    if not self.compile_function_error(token, e):
                        raise
                self.extend_nodelist(nodelist, compiled_result, token)
                self.exit_command()
        if parse_until:
            self.unclosed_block_tag(parse_until)
        return nodelist
    def parser_patch(instance):
        instance.__class__.parse = parse
        return instance
else:
    from django.template.defaulttags import IfNode
    parser_patch = lambda instance: instance




# vim: set fileencoding=utf-8 :
"""
Utilities


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement

__all__ = ('get_permission_codename', 'autodiscover',)

def get_permission_codename(perm):
    """get permission codename from permission string

    Usage:
        >>> get_permission_codename(u"app_label.codename_model")
        u"codename_model"
        >>> get_permission_codename(u"app_label.codename")
        u"codename"
        >>> get_permission_codename(u"codename_model")
        u"codename_model"
        >>> get_permission_codename(u"codename")
        u"codename"
        >>> get_permission_codename(u"app_label.app_label.codename_model")
        u"app_label.codename_model"

    """
    try:
        perm = perm.split('.', 1)[1]
    except IndexError:
        pass
    return perm

def autodiscover(module_name=None):
    """
    Auto-discover INSTALLED_APPS permissions.py modules and fail silently when
    not present. This forces an import on them to register any permissions bits
    they may want.

    """
    import copy
    from django.conf import settings
    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule
    from permission.handlers import registry

    module_name = module_name or settings.PERMISSION_MODULE_NAME

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)
        # Attempt to import the app's permissions module
        try:
            before_import_registry = copy.copy(registry._registry)
            import_module('%s.%s' % (app, module_name))
        except:
            # Reset the model registry to the state before tha last import as
            # this import will have to reoccur on the next request and this
            # could raise NotRegistered and AlreadyRegistered exceptions
            # (see #8254)
            registry._registry = before_import_registry

            # Decide whether to bubble up this error. If the app just
            # doesn't have an permissions module, we can ignore the error
            # attempting to import it, otherwise we want it to bubble up.
            if module_has_submodule(mod, module_name):
                raise



# vim: set fileencoding=utf-8 :
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.test import TestCase
from django.http import HttpResponse
from django.views.generic import View

from permission import registry
from permission.decorators.class_decorators import permission_required
from permission.tests.mock import MagicMock as Mock
from permission.tests.override_settings import with_apps

@with_apps('permission.tests.test_app')
class PermissionClassDecoratorsTestCase(TestCase):

    fixtures = ('permission_test_app.yaml',)

    def setUp(self):
        from permission.tests.test_app.models import Article
        mock_handler = Mock(**{
                'has_perm.return_value': False,
                'get_permissions.return_value': 'test_app.add_article',
            })
        mock_request = Mock()
        mock_request.META = Mock()
        mock_request.user = Mock()
        mock_request.user.is_active.return_value = True
        mock_request.user.is_authenticated.return_value = True
        mock_request.user.has_perm.side_effect = mock_handler.has_perm

        self.mock_handler = mock_handler
        self.mock_request = mock_request

        # clear registry and register mock handler
        registry._registry = {}
        registry._permissions = {}
        registry.register(
                Article,
                Mock(return_value=self.mock_handler)
            )
        self.view_func = Mock(return_value=HttpResponse)
        view_class = type('MockView', (View,), {})
        view_class.dispatch = self.view_func
        view_class = permission_required('test_app.add_article')(view_class)
        self.view_class = view_class

    def test_with_object(self):
        from permission.tests.test_app.models import Article
        self.assertEqual(registry._registry[Article], self.mock_handler)

        self.view_class.object = Article.objects.get(pk=1)
        # has_perm always return False
        self.view_class.as_view()(self.mock_request, pk=1)
        self.mock_request.user.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.mock_handler.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.assertFalse(self.view_func.called)
        # has_perm always return True
        self.mock_handler.has_perm.return_value = True
        self.view_class.as_view()(self.mock_request, pk=1)
        self.mock_request.user.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.mock_handler.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.assertTrue(self.view_func.called)
        del self.view_class.object

    def test_with_get_object(self):
        from permission.tests.test_app.models import Article
        self.assertEqual(registry._registry[Article], self.mock_handler)

        self.view_class.get_object = Mock(return_value=Article.objects.get(pk=1))
        # has_perm always return False
        self.view_class.as_view()(self.mock_request, pk=1)
        self.mock_request.user.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.mock_handler.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.view_class.get_object.assert_called_with(None)
        self.assertFalse(self.view_func.called)
        # has_perm always return True
        self.mock_handler.has_perm.return_value = True
        self.view_class.as_view()(self.mock_request, pk=1)
        self.mock_request.user.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.mock_handler.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.assertTrue(self.view_func.called)
        del self.view_class.get_object

    def test_with_queryset(self):
        from permission.tests.test_app.models import Article
        self.assertEqual(registry._registry[Article], self.mock_handler)

        self.view_class.queryset = "mock queryset"
        self.view_class.get_object = Mock(return_value=Article.objects.get(pk=1))
        # has_perm always return False
        self.view_class.as_view()(self.mock_request, pk=1)
        self.mock_request.user.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.mock_handler.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.view_class.get_object.assert_called_with(
                "mock queryset",
            )
        self.assertFalse(self.view_func.called)
        # has_perm always return True
        self.mock_handler.has_perm.return_value = True
        self.view_class.as_view()(self.mock_request, pk=1)
        self.mock_request.user.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.mock_handler.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.assertTrue(self.view_func.called)
        del self.view_class.get_object
        del self.view_class.queryset

    def test_with_get_queryset(self):
        from permission.tests.test_app.models import Article
        self.assertEqual(registry._registry[Article], self.mock_handler)

        self.view_class.get_queryset = Mock(return_value="mock queryset")
        self.view_class.get_object = Mock(return_value=Article.objects.get(pk=1))
        # has_perm always return False
        self.view_class.as_view()(self.mock_request, pk=1)
        self.mock_request.user.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.mock_handler.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.view_class.get_object.assert_called_with(
                "mock queryset",
            )
        self.assertFalse(self.view_func.called)
        # has_perm always return True
        self.mock_handler.has_perm.return_value = True
        self.view_class.as_view()(self.mock_request, pk=1)
        self.mock_request.user.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.mock_handler.has_perm.assert_called_with(
                'test_app.add_article',
                obj=Article.objects.get(pk=1)
            )
        self.assertTrue(self.view_func.called)
        del self.view_class.get_object
        del self.view_class.get_queryset

